package cz.autocont.iszr.demo;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Tato trida umoznuje zkonfigurovat demonstrace a zpust je z prikazove radky.
 * To, ktera demonstrace bude v spusteno, je definovano v konfiguracnim souboru.
 * Konfiguracni soubor je urceny parametrem {@link #CONFIG}. Tento soubor obsahuje klic {@link #CLASS}.
 * Pokud tento klic neni definovany, pak se spusti demonstrace {@link ReferentialAgent}.
 * 
 *  <pre>priklad konfigurace: -cfg=cfg\ISZR.properties</pre>
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class Execute
{
    /**
     * Atribut aplikace, ktery obsahuje konfiguracni soubor(y)
     */
    public static final String CONFIG = "-cfg";
    /**
     * Oddelovac jmena a hodnoty atributu aplikace
     */
    public static final char DELIM = '=';
    /**
     * Konfiguracni atribut, ktery obsahuje jmeno tridy dema. Toto demo bude spusteno.
     */
    public static final String CLASS = "class";
    /**
     * Odkaz na konfguracni soubor(y), ktery bude parsovany drive, nez aktualni soubor.
     */
    public static final String IMPORT_PREFIX = "@import=";
    /**
     * Oddelovat jednotlivych hodnot argumentu (jmen konfiguracnich souboru).
     */
    public static final String ARG_DELIM = ",";
    /**
     * Argumenty aplikace (jmeno, hodnota).
     */
    public static Map<String, String> args = new HashMap<>();
    /**
     * Konfigurace aplikace (jmeno, hodnota).
     */
    public static Properties config = new Properties();

    /**
     * Naparsuje parametry aplikace, naparsuje konfiguraci, provede setup prostredi a nakonec spusti demo.
     * 
     * @param args parametry
     * @see #parseArguments(String[])
     * @see #parseConfig()
     * @see #setup()
     * @see #execute()
     */
    public static void main(String[] args) throws Exception
    {
        Execute execute = new Execute();
        execute.parseArguments(args);
        execute.parseConfig();
        execute.setup();
        execute.execute();
    }

    /**
     * Provede naparsovani atributu aplikace. Vysledek bude ulozeny do {@link #args}.
     * 
     * @param args Atrybuty z prikazove radky aplikace.
     */
    public void parseArguments(String[] args)
    {
        for (String item : args)
        {
            // naparsovani atributu
            String key = item;
            String value = null;            
            // Rozdeleni atributu na klic a hodnotu
            int position = item.indexOf(DELIM);
            if (position > 0)
            {
                key = item.substring(0, position);
                value = item.substring(position + 1);
            }
            // Ulozeni atributu do konficurate
            this.getArgs().put(key, value);
        }
    }

    /**
     * Provede naparsovani konfigurace. Konfigurace je ulozena pod atributem {@link #CONFIG}.
     * Vysledek parsovani bude ulozeny do {@link #config}
     * 
     * @throws IOException Pokud soubor neexistuje, nebo neni mozne z neho cist.
     * @see #parseConfig(String)
     */
    public void parseConfig() throws IOException
    {
        //Ziskani definice konfigurace
        String configFile = this.getArgs().get(CONFIG);

        if (configFile != null)
        {
            // ziskani jednotlivych jmen konfiguracnich souboru
            String[] files = configFile.split(ARG_DELIM);
            // Parsovani jednotlivych konfiguracnich souboru
            for (String item : files)
            {
                parseConfig(item);
            }
        }
    }

    /**
     * Provede naparsuje dany konfiguracni soubor. Nejdrive naparsuje soubory z predka a pak aktualni soubor.
     * Vysledek parsovani bude ulozeny do {@link #config}. Konfiguracni soubor je v kodovani ISO-8859-1.
     * 
     * @param configFile Soubor, ktery bude naparsovan.
     * @throws IOException Pokud soubor neexistuje, nebo neni mozne z neho cist.
     * @see #parseParentConfig(String)
     */
    public void parseConfig(String configFile) throws IOException
    {
        // Naparsovani predka konfiguracniho souboru
        parseParentConfig(configFile);
        // Naparsovani daneho konfiguracniho souboru
        try (InputStreamReader in = new InputStreamReader(new FileInputStream(configFile), StandardCharsets.ISO_8859_1))
        {
            this.getConfig().load(in);
        }
    }

    /**
     * Provede naparsovani souboru predka. Predek je uvedeny na prvnim radku souboru pod klicem {@link #IMPORT_PREFIX}.
     * 
     * @param currentConfig Soubor, jehoz predek bude naparsovan.
     * @throws IOException Pokud soubor neexistuje, nebo neni mozne z neho cist.
     * @see #parseConfig(String)
     */
    public void parseParentConfig(String currentConfig) throws IOException
    {
        // Precteni prvniho radku konfiguracniho souboru
        BufferedReader br = new BufferedReader(new FileReader(currentConfig));
        String firstLine;
        try
        {
            firstLine = br.readLine();
        }
        finally
        {
            br.close();
        }
        // test prvniho radku
        if (firstLine != null && firstLine.startsWith(IMPORT_PREFIX))
        {
            // Prvni radek obsahuje odkaz na predky
            // Ziskani jmen souboru
            String[] parents = firstLine.substring(IMPORT_PREFIX.length()).split(ARG_DELIM);
            // Naparsovani jednotlivych predku
            for (String item : parents)
            {
                parseConfig(item);
            }
        }
    }

    /**
     * Provede setup prostredi.
     * Tato implementace nastavi vsechny property z {@link #config}, jejich kliz zacina na -D, do systemovych promennych (
     * {@link System#setProperty(String, String)}).
     */
    public void setup()
    {
        // Zkopirovani vstupnich argumentu do konfigurace -> pretizeni konfigurace
        config.putAll(args);
        
        // Nastaveni systemovych atributu, ktere jsou obsazeny v konfiguraci
        for (Map.Entry<Object, Object> item : getConfig().entrySet())
        {
            String key = (String) item.getKey();
            if (key.startsWith("-D"))
            {
                String value = (String) item.getValue();
                System.setProperty(key.substring(2), value);
            }
        }
    }

    /**
     * Spusti demonstraci. <br/>
     * Jmeno tridy, ktera bude spustena, je uvedeno v {@link #config} pod klicem {@link #CLASS}.
     * 
     * @throws ClassNotFoundException - tridu neni mozne vytvorit.
     * @throws IllegalArgumentException - tridu neni mozne vytvorit
     * @throws InstantiationException - tridu neni mozne vytvorit.
     * @throws IllegalAccessException - tridu neni mozne vytvorit.
     * @throws InvocationTargetException - tridu neni mozne vytvorit.
     * @throws DemoException - nastala chyba v demu
     */
    public void execute() throws ClassNotFoundException, IllegalArgumentException, InstantiationException, IllegalAccessException,
            InvocationTargetException, DemoException
    {
        // Ziskani jmena tridy, ktera implementuje demostraci
        String className = getConfig().getProperty(CLASS, ReferentialAgent.class.getName());
        System.out.println("Execute demo: " + className);

        // Vytvoreni demostrace
        @SuppressWarnings("unchecked")
        Class<Demo> clazz = (Class<Demo>) getClass().getClassLoader().loadClass(className);
        Demo demo;
        try
        {
            Constructor<Demo> constructor = clazz.getConstructor(Map.class, Properties.class);
            demo = constructor.newInstance(getArgs(), getConfig());
        } catch (NoSuchMethodException e)
        {
            demo = clazz.newInstance();
        }

        // Spusteni demostrace
        demo.execute();
    }

    /**
     * @return the args
     */
    public Map<String, String> getArgs()
    {
        return args;
    }

    /**
     * @return the config
     */
    public Properties getConfig()
    {
        return config;
    }
}
