package cz.autocont.iszr.demo.e256;

import cz.autocont.iszr.demo.BaseDemoCall;
import cz.autocont.iszr.demo.DemoException;
import cz.isvs.iszr.schemas.iszrabstract.v1.AutorizaceType;
import cz.isvs.iszr.schemas.iszrabstract.v1.ZadostInfoIszrType;
import cz.isvs.iszr.schemas.iszrrosctiico2.v1.RosCtiIco2ResponseType;
import cz.isvs.iszr.schemas.iszrrosctiico2.v1.RosCtiIco2Type;
import cz.isvs.iszr.services.iszrrosctiico2.v1.IszrRosCtiIco2PortType;
import cz.isvs.iszr.services.iszrrosctiico2.v1.IszrRosCtiIco2Service;
import cz.isvs.reg.schemas.regtypy.v1.VysledekKodType;
import cz.isvs.ros.schemas.rosdotazydata.v2.RosCtiIco2DataType;

import javax.xml.namespace.QName;
import jakarta.xml.ws.BindingProvider;
import jakarta.xml.ws.WebServiceClient;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

/**
 * Demostrace synchronniho volani webove sluzby E256 - IszrRosCtiIco.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class DemoCallE256 extends BaseDemoCall<RosCtiIco2Type, RosCtiIco2ResponseType>
{

    /**
     * Vytvori data, ktera budou odeslana do webove sluzby.
     * 
     * @return Odesilana data.
     */
    @Override
    protected RosCtiIco2Type createRequestData()
    {
        RosCtiIco2Type result = new RosCtiIco2Type();

        result.setZadostInfo(createZadostInfo());
        result.setAutorizaceInfo(createAutorizaceInfo());
        result.setZadost(createRosCtiIcoTypeZadost());

        return result;
    }

    /**
     * Zavola webovou sluzbu E256.
     * 
     * @param requestData Data, ktera budou odeslana na webovou sluzbu.
     * @return Data, ktera vratila webova sluzba.
     * @throws DemoException Pokud pri volani sluzby dojde k chybe.
     */
    @Override
    protected RosCtiIco2ResponseType callService(RosCtiIco2Type requestData) throws DemoException
    {
        // get service port
        IszrRosCtiIco2PortType port = createServicePort();

        System.out.println("Pro komunikaci budou pouzity tyto hodnoty:");
        System.out.println("Agenda=" + requestData.getZadostInfo().getAgenda());
        System.out.println("Role=" + requestData.getZadostInfo().getAgendovaRole());
        System.out.println("Ais=" + requestData.getZadostInfo().getAis());
        System.out.println("Ovm=" + requestData.getZadostInfo().getOvm());
        String target = (String) ((BindingProvider) port).getRequestContext().get(BindingProvider.ENDPOINT_ADDRESS_PROPERTY);
        System.out.println("Probiha asynchronni volani webove sluzby " + target);

        // call service

        return port.iszrRosCtiIco2(requestData);
    }

    /**
     * Provede zpracovani dat, ktera vatila webova sluzba.
     * IszrZadostId, ktere je ulozeno v odpovedi, ulozi do konfigurace pod klicem "E256.iszrZadostId".
     * 
     * @param responseData Data, ktera vratila wbova sluzba.
     */
    @Override
    protected void processResponseData(RosCtiIco2ResponseType responseData)
    {
        VysledekKodType resultCode = responseData.getOdpovedInfo().getStatus().getVysledekKod();
        String iszrId = responseData.getOdpovedInfo().getIszrZadostId();

        System.out.println("Pridelene IszrZadostId:" + iszrId);
        System.out.println("Vysledek:" + resultCode);

        // Ulozeni ISZR zadost ID
        getConfig().put("E100.iszrZadostId", iszrId);
    }

    /**
     * Spolecna hlavicka vsech externich zadosti nebo dotazu (webovych sluzeb). <br/>
     * Vytvori zakladn strukturu zadosti pro sluzbu ISZR. {@link ZadostInfoIszrType} obsahuje informace
     * o volajici strane a danem pozadavku. Na zaklade techto dat ISZR rozhodne, o tom, jestli ma volajici strana pravo vykonat volanou sluzbu.
     * 
     * @return Vyplnene informace o volajici strane.
     */
    @Override
    protected ZadostInfoIszrType createZadostInfo()
    {
        ZadostInfoIszrType result = super.createZadostInfo();

        result.setSubjekt(getConfig().getProperty("E256.subjekt"));
        result.setUzivatel(getConfig().getProperty("E256.uzivatel"));
        result.setDuvodUcel(getConfig().getProperty("E256.duvodUcel"));

        return result;
    }

    /**
     * Seznam autorizacnich udaju.
     * 
     * @return seznam hodnot.
     */
    protected AutorizaceType createAutorizaceInfo()
    {
        AutorizaceType autorizaceType = new AutorizaceType();
        String[] values = getConfig().getProperty("E256.autorizace.udaje", "").split("[\\s]*\\|[\\s]*");
        for (String item : values)
        {
            autorizaceType.getSeznamUdaju().add(item);
        }

        return autorizaceType;
    }

    /**
     * Vytvori datovou cast pozadavku.
     * 
     * @return Data pozadavku
     */
    protected RosCtiIco2Type.Zadost createRosCtiIcoTypeZadost()
    {
        RosCtiIco2Type.Zadost result = new RosCtiIco2Type.Zadost();
        result.setRosCtiIco2Data(new RosCtiIco2DataType());
        String icoStr = getConfig().getProperty("E256.ico");
        result.getRosCtiIco2Data().setIco(Integer.parseInt(icoStr));
        return result;
    }

    /**
     * Vytvori proxy port na webovou sluzbu.
     * 
     * @return Proxy port
     * @throws DemoException Pokud pri vytvareni portu dojde k chybe.
     */
    private IszrRosCtiIco2PortType createServicePort() throws DemoException
    {
        WebServiceClient webServiceClient = IszrRosCtiIco2Service.class.getAnnotation(WebServiceClient.class);
        final String TARGET_ENDPOINT_ADDRESS = MessageFormat.format(getConfig().getProperty("iszr.location"), "E256?async");
        final String TARGET_SERVICE_NAMESPACE = webServiceClient.targetNamespace();
        final String TARGET_SERVICE_NAME = webServiceClient.name();
        final String TARGET_WSDL_LOCATION = MessageFormat.format(getConfig().getProperty("wsdl.location"), "IszrRosCtiIco2.wsdl");

        // get service
        IszrRosCtiIco2Service service;
        try
        {
            service = new IszrRosCtiIco2Service(new URL(TARGET_WSDL_LOCATION), new QName(TARGET_SERVICE_NAMESPACE, TARGET_SERVICE_NAME));
        } catch (MalformedURLException e)
        {
            throw new DemoException("Wrong target address: " + TARGET_ENDPOINT_ADDRESS, e);
        }
        IszrRosCtiIco2PortType port = service.getIszrRosCtiIco2PortService();

        // setup target adress and SOAP loging
        setUpServicePort(port, "E256", TARGET_ENDPOINT_ADDRESS);

        return port;
    }
}
