package cz.autocont.iszr.demo.log;

import jakarta.xml.ws.handler.MessageContext;
import jakarta.xml.ws.handler.soap.SOAPMessageContext;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;


/**
 * SOAPHandler, ktery provede tisk SOAP zpravy to souboru.
 * Pro kazdou zpravu vytvori novy soubor. Jmeno souboru se zklada z kodu vebove sluzby,
 * datumu casu a slova (request/response). Soubory jsou ulozeny v adresari,
 * ktery je devinovany v konfiguraci {@link #config} pod klicem "soap.log.location".
 *
 * <pre>
 * Priklad jmena souboru: E256_2012-03-08-113352_request.xml
 * </pre>
 *
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 *
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class SOAPFileLoggingHandler extends SOAPLoggingHandler
{

    /**
     * Kod webove sluzby. Je soucasti jmena souboru.
     */
    private final String serviceNo;

    /**
     * Konfigurace.
     */
    private final Properties config;

    /**
     * Formater pro formatovani casu a datumu. Vystup je soucasti jmena souboru.
     */
    private final DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HHmmss");

    /**
     * @param serviceNo {@link #serviceNo}
     * @param config    {@link #config}
     */
    public SOAPFileLoggingHandler(String serviceNo, Properties config)
    {
        super();
        this.serviceNo = serviceNo;
        this.config = config;
    }

    /**
     * Provede tisk SOAP zpravy do souboru.<br/>
     * <ul>
     * <li>Vytvori novy soubor. Jmeno souboru sestavi na zaklade {@link #serviceNo}, aktualniho datumu a casu a informaci,
     * jestli se jedna o odhozi, nebo prihozi zpravu (MessageContext#MESSAGE_OUTBOUND_PROPERTY).</li>
     * <li>Tento soubor vlozi do {@link #setOut(PrintStream)}.</li>
     * <li>Pote provede tisk SOAP zpravy volanim {@link SOAPLoggingHandler#logSoapMessage(SOAPMessageContext)}</li>
     * <li>Nakonec vrati do {@link #setOut(PrintStream)} puvodni {@link PrintStream}</li>
     * </ul>
     *
     * @param smc SOAP kontext.
     * @throws IllegalStateException Pokud nelze otevrit vystupni soubor, nebo tisknout do tohoto souboru.
     * @see SOAPLoggingHandler#logSoapMessage(SOAPMessageContext)
     */
    @Override
    protected void logSoapMessage(SOAPMessageContext smc)
    {
        // Vytvoreni adresare pro ukladani souboru.
        String dir = getConfig().getProperty("soap.log.location");
        File fdir = new File(dir);
        fdir.mkdirs();

        // Vytvoreni jmena souboru
        String name = dateFormat.format(new Date());
        Boolean outboundProperty = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        if (outboundProperty)
        {
            name = getServiceNo() + "_" + name + "_request.xml";
        } else
        {
            name = getServiceNo() + "_" + name + "_response.xml";
        }

        // Vytvoreni souboru pro ulozeni SOAP zpravy
        String pathname = dir + "/" + name;
        File file = new File(pathname);

        // Tisk SOAP zpravy
        synchronized (this)
        {
            // Vytvoreni souboru na disku
            PrintStream out = null;
            try
            {
                out = new PrintStream(file);
            } catch (FileNotFoundException e)
            {
                throw new IllegalStateException("Can't open SOAP log file: " + pathname, e);
            }
            try
            {
                // Nastaveni noveho vystupu
                PrintStream oldOut = getOut();
                setOut(out);
                // Tisk zpravy
                super.logSoapMessage(smc);
                out.flush();
                out.close();
                // Nastaveni puvodniho vystupu
                setOut(oldOut);
            } catch (Exception e)
            {
                throw new IllegalStateException("Can't log SOAP message into file log file: " + pathname, e);
            }
        }

        // Tisk informace o logovacim souboru
        if (outboundProperty)
        {
            System.out.println("Vstupni pozadavek byl ulozen do souboru: " + pathname);
        } else
        {
            System.out.println("Vysledna odpoved byla ulozena do souboru: " + pathname);
        }
    }

    /**
     * @return the serviceNo
     */
    public String getServiceNo()
    {
        return serviceNo;
    }

    /**
     * @return the config
     */
    public Properties getConfig()
    {
        return config;
    }

}
